#property copyright "Portfolio Optimizer - by transcendreamer"
#include <Math\Alglib\alglib.mqh>
#property indicator_separate_window
#property indicator_buffers 1
#property indicator_level1 0

extern datetime start_eqity=D'2014.01.01 00:00'; // Start eqity
extern datetime start_time=D'2014.08.13 12:00'; // Start time
extern datetime finish_time=D'2014.11.13 16:00'; // Finish time
extern ENUM_TIMEFRAMES timeframe=PERIOD_H1; // Timeframe
extern string symbol_list="EURUSD GBPUSD AUDUSD NZDUSD"; // Portfolio formula
enum regression {trend,oscillator,spread};
extern regression model_type=spread; // Model type
extern double trade_lots=1; // Lots to trade
extern int lots_digits=2; // Lots digits
extern string chart_currency="USD"; // Chart currency
extern bool invert_chart=false; // Invert chart
extern bool show_value=false; // Show Bid/Ask
extern double grid_size=0; // Chart grid size
extern color chart_color=clrLime; // Chart color
extern string export_file=""; // CSV export file
extern string delimiter=";"; // CSV delimiter

long chart;
int window;
string SYMBOLS[];
int total,points;
double opening[],closing[],profit[];
double EQUITY[][100],MODEL[],SYNTH[],ROOT[],LOTS[];
double portfolio[];
datetime TIMES[];
datetime current_time,zero_time;
bool error=false,formula=false,first_run=true;
double spread_sum,margin_sum;
datetime tick;

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void OnInit()
{
IndicatorDigits(2);
IndicatorShortName("Portfolio Equity");
SetIndexBuffer(0,portfolio);
SetIndexStyle(0,DRAW_LINE,STYLE_SOLID,1,chart_color);
SetIndexLabel(0,"Portfolio Equity");
ArrayInitialize(portfolio,EMPTY_VALUE);
window=ChartWindowFind();
chart=ChartID();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

int OnCalculate(const int rates_total, const int prev_calculated, 
const datetime& time[], const double& open[], const double& high[],
const double& low[], const double& close[],const long& tick_volume[],
const long& volume[], const int& spread[])
{
if(first_run) DefineSymbols();
if(first_run) DefineFormula();
if(first_run) CheckSymbols();
if(error) return(0);
if(first_run) DefineZeroPoint();
if(first_run) CalculateEquity();
if(first_run) CalculateRegression();
if(first_run) NormalizeLots();
if(first_run) ExportEquityData();
if(first_run) DisplayFormula();
if(error) return(0);
ChartPortfolio(rates_total-prev_calculated+1);
UpdateStatus();
if(first_run) CalculateRange();
if(first_run) DisplayGrid();
first_run=false;
return(rates_total);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void OnDeinit(const int reason)
{
int objects=ObjectsTotal();
for(int i=objects; i>=0; i--)
   {
   string name=ObjectName(i);
   bool test1=(StringFind(name,"Formula-label-"+(string)window)!=-1);
   bool test2=(StringFind(name,"Out-of-sample-"+(string)window)!=-1);
   bool test3=(StringFind(name,"Grid-level-"+(string)window)!=-1);
   bool test4=(StringFind(name,"Info-label-"+(string)window)!=-1);
   bool test5=(StringFind(name,"Portfolio-Bid-"+(string)window)!=-1);
   bool test6=(StringFind(name,"Portfolio-Ask-"+(string)window)!=-1);
   if(test1||test2||test3||test4||test5||test6) 
      if(ObjectFind(name)==window) ObjectDelete(name);
   }
GlobalVariableDel("Equity-Bid-"+(string)chart+"-"+(string)window);
GlobalVariableDel("Equity-Ask-"+(string)chart+"-"+(string)window);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void DefineSymbols()
{
total=0;
string name="";
int length=StringLen(symbol_list);
for(int position=0; position<length; position++)
   {
   string X=StringSubstr(symbol_list,position,1);
   if(X=="+"||X=="-") formula=true;
   if(X!=" ") name=name+X;
   if((X==" "||position==length-1)&&(StringLen(name)>0))
      {
      total++;
      ArrayResize(SYMBOLS,total);
      ArrayResize(opening,total);
      ArrayResize(closing,total);
      ArrayResize(profit,total);
      ArrayResize(ROOT,total);
      ArrayResize(LOTS,total);
      SYMBOLS[total-1]=name;
      name="";
      }
   }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void DefineFormula()
{
if(!formula) return;
for(int i=0; i<total; i++)
   {
   int length=StringLen(SYMBOLS[i]);
   int position=length-1;
   while(position>=0)
      {
      string X=StringSubstr(SYMBOLS[i],position,1);
      if(X=="+"||X=="-") break;
      if(position==0) break;
      else position--;
      }
   if(position==0) LOTS[i]=1;
   else LOTS[i]=StrToDouble(StringSubstr(SYMBOLS[i],position,length-position));
   SYMBOLS[i]=StringSubstr(SYMBOLS[i],0,position);
   }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void CheckSymbols()
{
if(total==0) { error=true; Alert("Empty portfolio formula!"); return; }
for(int i=0; i<total; i++) if(MarketInfo(SYMBOLS[i],MODE_POINT)==0)
{ error=true; Alert("Missing symbol! - ",SYMBOLS[i]); }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void DefineZeroPoint()
{
current_time=start_time;
bool seek_history=true;
while(seek_history && current_time<=iTime(Symbol(),timeframe,0) && current_time<=finish_time)
   {
   seek_history=false;
   for(int i=0; i<total; i++)
      {
      int shift=iBarShift(SYMBOLS[i],timeframe,current_time,true);
      if(shift==-1) seek_history=true;
      else opening[i]=iClose(SYMBOLS[i],timeframe,shift);
      }
   if(seek_history) current_time+=timeframe*60;
   }
zero_time=current_time;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void CalculateEquity()
{
if(formula) return;
int shift,i;
points=0;
current_time=zero_time;
while(current_time<=iTime(Symbol(),timeframe,0) && current_time<=finish_time)
   {
   bool skip_step=false;
   for(i=0; i<total; i++)
      if(iBarShift(SYMBOLS[i],timeframe,current_time,true)==-1)
         skip_step=true;
   if(!skip_step)
      {points++;
      ArrayResize(EQUITY,points);
      ArrayResize(TIMES,points);
      TIMES[points-1]=current_time;
      for(i=0; i<total; i++)
         {shift=iBarShift(SYMBOLS[i],timeframe,current_time,true);
         closing[i]=iClose(SYMBOLS[i],timeframe,shift);
         double PV=PointValue(SYMBOLS[i],current_time,timeframe);
         profit[i]=(closing[i]-opening[i])*PV;
         EQUITY[points-1,i]=profit[i];}}
   current_time+=timeframe*60;
   }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void CalculateRegression()
{
if(formula) return;
int info,i,j,roots;
double scale=0;
ArrayResize(SYNTH,points);
ArrayResize(MODEL,points);
CLinearModelShell LM;
CLRReportShell AR;
for(i=0; i<total; i++) scale+=PointValue(SYMBOLS[i],Time[0],timeframe);
if(model_type==trend) for(j=0; j<points; j++) MODEL[j]=j*scale;
if(model_type==oscillator) for(j=0; j<points; j++) MODEL[j]=scale*(2*(j%2)-1);
if(model_type==spread) roots=total-1; else roots=total;
CMatrixDouble MATRIX(points,roots+1);
for(i=0; i<total; i++) for(j=0; j<points; j++) MATRIX[j].Set(i,EQUITY[j,i]);
if(model_type!=spread) for(j=0; j<points; j++) MATRIX[j].Set(total,MODEL[j]);
CAlglib::LRBuildZ(MATRIX,points,roots,info,LM,AR);
if(info!=1) {Alert("Error in regression model!");error=true;return;}
CAlglib::LRUnpack(LM,ROOT,roots);
ArrayResize(ROOT,total);
if(model_type==spread) ROOT[total-1]=-1;
for(j=0; j<points; j++) for(i=0; i<total; i++) SYNTH[j]+=ROOT[i]*EQUITY[j,i];
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void NormalizeLots()
{
if(formula) return;
double sum_abs=0;
ArrayResize(LOTS,total);
for(int i=0; i<total; i++) sum_abs+=MathAbs(ROOT[i]);
for(int i=0; i<total; i++) LOTS[i]=NormalizeDouble(ROOT[i]*trade_lots/sum_abs,lots_digits);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void ChartPortfolio(int bars)
{
double sum=0;
int open_bar,close_bar;
double sign=1; if(invert_chart) sign=-1;
open_bar=iBarShift(Symbol(),Period(),start_eqity);
if(bars<open_bar+1) open_bar=bars-1;
if(!formula) close_bar=0; else close_bar=iBarShift(Symbol(),Period(),finish_time);
for(int j=open_bar; j>=close_bar; j--)
   {
   sum=0;
   for(int i=0; i<total; i++)
      {
      int shift=iBarShift(SYMBOLS[i],Period(),Time[j]);
      closing[i]=iClose(SYMBOLS[i],Period(),shift);
      double PV=PointValue(SYMBOLS[i],Time[j],Period());
      profit[i]=(closing[i]-opening[i])*PV*LOTS[i];
      sum+=sign*profit[i];
      }
   portfolio[j]=NormalizeDouble(sum,2);
   }
if(Time[0]>=finish_time && !formula)
   PlaceVertical("Out-of-sample-finish "+(string)window,finish_time,Red,STYLE_DOT);
if(Time[0]>=start_time && !formula)
   PlaceVertical("Out-of-sample-start  "+(string)window,start_time,clrAqua,STYLE_DOT);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void CalculateRange()
{
string text;
int open_bar,close_bar;
double max=-999999999999, min=999999999999;
open_bar=iBarShift(Symbol(),Period(),zero_time);
if(!formula) close_bar=0; else close_bar=iBarShift(Symbol(),Period(),finish_time);
double growth=MathAbs(portfolio[open_bar]-portfolio[close_bar])/(open_bar-close_bar);
if(model_type!=trend) for(int j=open_bar; j>=close_bar; j--)
   { 
   if(portfolio[j]>max) max=portfolio[j]; 
   if(portfolio[j]<min) min=portfolio[j];
   }
if(model_type==trend) for(int j=open_bar; j>=close_bar; j--)
   {
   if(portfolio[j]-growth*(open_bar-j)>max) max=portfolio[j]-growth*(open_bar-j);
   if(portfolio[j]-growth*(open_bar-j)<min) min=portfolio[j]-growth*(open_bar-j);
   }
text="Volatility: "+DoubleToString(max-min,2)+" "+chart_currency;
PlaceLabel("Data-label-"+(string)window,5,15,CORNER_RIGHT_LOWER,text,chart_color,"Tahoma",8);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

double PointValue(string symbol, datetime time, int period)
{
double value=0;
double close1,close2;
string basis;
if(chart_currency=="") chart_currency="USD";
int type=(int)MarketInfo(symbol,MODE_PROFITCALCMODE);
switch(type)
{
case 0:
   {
   int shift1=iBarShift(symbol,period,time);
   value=MarketInfo(symbol,MODE_LOTSIZE);
   if(StringSubstr(symbol,3,3)=="USD") break;
   if(StringSubstr(symbol,0,3)=="USD")
      {close1=iClose(symbol,period,shift1);
      if(close1>0) value=value/close1;}
   else
      {
      basis=StringSubstr(symbol,0,3)+"USD";
      if(iClose(basis,period,0)==0) basis="USD"+StringSubstr(symbol,0,3);
      if(iClose(basis,period,0)==0) break;
      int shift2=iBarShift(basis,period,time);
      close1=iClose(symbol,period,shift1);
      close2=iClose(basis,period,shift2);
      if(close1>0 && close2>0)
         {if(StringSubstr(basis,0,3)=="USD") value=value/close2/close1;
         else value=value*close2/close1;}
      }
   } break;
case 1: value=MarketInfo(symbol,MODE_LOTSIZE); break;
case 2: value=MarketInfo(symbol,MODE_TICKVALUE)/MarketInfo(symbol,MODE_TICKSIZE);
}
if(chart_currency!="USD")
   {
   basis=chart_currency+"USD";
   if(iClose(basis,period,0)==0)
      {basis="USD"+chart_currency;
      close1=iClose(basis,period,iBarShift(basis,period,time));
      if(close1>0) value*=close1;}
   else
      {close1=iClose(basis,period,iBarShift(basis,period,time));
      if(close1>0) value/=close1;}
   }
return(value);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void UpdateStatus()
{
string text;
spread_sum=0;
margin_sum=0;
for(int i=0; i<total; i++)
   {
   double PV=PointValue(SYMBOLS[i],Time[0],Period());
   double pips=MarketInfo(SYMBOLS[i],MODE_ASK)-MarketInfo(SYMBOLS[i],MODE_BID);
   spread_sum+=pips*PV*MathAbs(LOTS[i]);
   margin_sum+=MarketInfo(SYMBOLS[i],MODE_MARGINREQUIRED)*MathAbs(LOTS[i]);
   }
spread_sum=NormalizeDouble(spread_sum,2);
margin_sum=NormalizeDouble(margin_sum,2);
text="Spread: "+DoubleToString(spread_sum,2)+" "+chart_currency+"    "+
"Margin: "+DoubleToString(margin_sum,2)+" "+AccountCurrency();
PlaceLabel("Info-label-"+(string)window,5,3,CORNER_RIGHT_LOWER,text,chart_color,"Tahoma",8);
if(show_value)
   {
   ObjectDelete("Portfolio-Bid-"+(string)window);
   ObjectDelete("Portfolio-Ask-"+(string)window);
   PlaceHorizontal("Portfolio-Bid-"+(string)window,portfolio[0],Red,STYLE_DOT);
   PlaceHorizontal("Portfolio-Ask-"+(string)window,portfolio[0]+spread_sum,Red,STYLE_DOT);
   }
GlobalVariableSet("Equity-Bid-"+(string)chart+"-"+(string)window,portfolio[0]);
GlobalVariableSet("Equity-Ask-"+(string)chart+"-"+(string)window,portfolio[0]+spread_sum);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void DisplayFormula()
{
string text,name;
double sign=1; if(invert_chart) sign=-1;
for(int i=0; i<total; i++)
   {
   text=SYMBOLS[i];
   if(sign*LOTS[i]>=0) text=text+"+"; else text=text+"-";
   text=text+DoubleToString(MathAbs(LOTS[i]),lots_digits);
   name="Formula-label-"+(string)window+"-"+(string)i;
   PlaceLabel(name,5,15+i*12,CORNER_LEFT_UPPER,text,Red,"Tahoma",8);
   }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void DisplayGrid()
{
if(grid_size==0) return;
int open_bar,close_bar;
double max=-2147483647; double min=2147483647;
open_bar=iBarShift(Symbol(),Period(),zero_time);
if(!formula) close_bar=0; else close_bar=iBarShift(Symbol(),Period(),finish_time);
for(int j=open_bar; j>=close_bar; j--)
   {if(portfolio[j]>max) max=portfolio[j]; 
   if(portfolio[j]<min) min=portfolio[j];}
double y=0; while(y<max) 
   {y+=grid_size;
   string name="Grid-level-"+(string)window+":"+DoubleToString(y,2);
   PlaceHorizontal(name,y,Silver,STYLE_DOT);}
y=0; while(y>min) 
   {y-=grid_size;
   string name="Grid-level-"+(string)window+":"+DoubleToString(y,2);
   PlaceHorizontal(name,y,Silver,STYLE_DOT);}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void PlaceLabel(string name, int x, int y, int corner, string text, int colour, string font, int size)
{
ObjectCreate(name,OBJ_LABEL,window,0,0);
ObjectSet(name,OBJPROP_CORNER,corner);
ObjectSet(name,OBJPROP_XDISTANCE,x); 
ObjectSet(name,OBJPROP_YDISTANCE,y);
ObjectSetText(name,text,size,font,colour);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void PlaceVertical(string name, datetime time, int colour, int style)
{
ObjectCreate(0,name,OBJ_VLINE,window,time,0);
ObjectSetInteger(0,name,OBJPROP_COLOR,colour);
ObjectSetInteger(0,name,OBJPROP_STYLE,style);
ObjectSetInteger(0,name,OBJPROP_BACK,true);
ObjectSetInteger(0,name,OBJPROP_SELECTABLE,false);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void PlaceHorizontal(string name, double price, int colour, int style)
{
ObjectCreate(0,name,OBJ_HLINE,window,0,price);
ObjectSetInteger(0,name,OBJPROP_COLOR,colour);
ObjectSetInteger(0,name,OBJPROP_STYLE,style);
ObjectSetInteger(0,name,OBJPROP_BACK,true);
ObjectSetInteger(0,name,OBJPROP_SELECTABLE,false);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void ExportEquityData()
{
string text;
if(export_file!=""){
export_file=export_file+".csv";
PrepareFile(export_file);
for(int j=0; j<points; j++)
{text=TimeToStr(TIMES[j],TIME_DATE|TIME_MINUTES);
for(int i=0; i<total; i++) text=StringConcatenate(text,delimiter,EQUITY[j,i]);
text=StringConcatenate(text,delimiter,SYNTH[j]);
WriteLine(export_file,text);}
Alert("Equity data export finished.");
}}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void PrepareFile(string file)
{
int handle=FileOpen(file,FILE_WRITE,delimiter);
if(handle==-1) {Alert("Error opening file!");error=true;}
FileClose(handle);
string text="DATE/TIME";
for(int i=0; i<total; i++) text=StringConcatenate(text,delimiter,SYMBOLS[i]);
text=StringConcatenate(text,delimiter,"PORTFOLIO");
WriteLine(file,text);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////

void WriteLine(string file, string text)
{
int handle=FileOpen(file,FILE_READ|FILE_WRITE,delimiter);
if(handle>0){
FileSeek(handle,0,SEEK_END);
FileWrite(handle,text);
FileClose(handle);}
}
